/*
Copyright (c) 1997-2005, Damon Hart-Davis
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are
met:

  * Redistributions of source code must retain the above copyright
    notice, this list of conditions and the following disclaimer.

  * Redistributions in binary form must reproduce the above copyright
    notice, this list of conditions and the following disclaimer in the
    documentation and/or other materials provided with the
    distribution.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/
package org.hd.d.tmf;

import javax.servlet.ServletInputStream;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletRequestWrapper;
import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;

/**Wrap the input for monitoring.
 * Mainly used to make estimation of non-I/O load more accurate.
 */
public final class ThroughputMonitorRequestWrapper extends HttpServletRequestWrapper
    {
    private ServletInputStream stream;
    private BufferedReader reader;
    private ThroughputMonitorFilterBase filter;
    private HttpServletRequest origRequest;

    public ThroughputMonitorRequestWrapper(ThroughputMonitorFilterBase f,
                                           HttpServletRequest request)
        {
        super(request);
        filter = f;
        origRequest = request;
        }

    private ServletInputStream createInputStream()
        throws IOException
        { return(new ThroughputMonitorRequestStream(filter, origRequest)); }

    public ServletInputStream getInputStream()
        throws IOException
        {
        if(reader != null) { throw new IllegalStateException("getReader() has already been called for this request"); }
        if(stream == null) { stream = createInputStream(); }
        return(stream);
        }

    public BufferedReader getReader()
        throws IOException
        {
        if(reader != null) { return(reader); }
        if(stream != null) { throw new IllegalStateException("getInputStream() has already been called for this request"); }
        stream = createInputStream();
        reader = new BufferedReader(new InputStreamReader(stream));
        return(reader);
        }

    public void finishRequest()
        throws IOException
        {
        final boolean didDecrement = filter.adjustBusyConnectionCount(false);
        try
            {
            if(reader != null)
                { reader.close(); }
            else if(stream != null)
                { stream.close(); }
            }
        finally { if(didDecrement) { filter.adjustBusyConnectionCount(true); } }
        }
    }
